----------------
-----Models-----
----------------

local E_MODEL_CHECKPOINT = smlua_model_util_get_id("Checkpoint_Flag_MOP")
local E_MODEL_CHECKPOINT2 = smlua_model_util_get_id("Checkpoint_Flag_MOP2")

--------------------
-----Collisions-----
--------------------

-----------------------------
-----Localized Functions-----
-----------------------------

local play_sound = play_sound
local spawn_non_sync_object = spawn_non_sync_object
local obj_set_model_extended = obj_set_model_extended
local obj_scale = obj_scale

--------------------------
-----Helper Functions-----
--------------------------

---@param m MarioState
---@return boolean
local function is_bubbled(m)
    return m.action == ACT_BUBBLED
end

--- @param obj Object
--- @param hitbox ObjectHitbox
function obj_set_hitbox(obj, hitbox)
	if not obj or not hitbox then return end
	-- Sets other hitbox values once
	if (obj.oFlags & OBJ_FLAG_30) == 0 then
		obj.oFlags = obj.oFlags | OBJ_FLAG_30

		obj.oInteractType = hitbox.interactType
		obj.oDamageOrCoinValue = hitbox.damageOrCoinValue
		obj.oHealth = hitbox.health
		obj.oNumLootCoins = hitbox.numLootCoins

		cur_obj_become_tangible()
	end

	-- Set actual hitboxes
	obj.hitboxRadius = obj.header.gfx.scale.x * hitbox.radius
	obj.hitboxHeight = obj.header.gfx.scale.y * hitbox.height
	obj.hurtboxRadius = obj.header.gfx.scale.x * hitbox.hurtboxRadius
	obj.hurtboxHeight = obj.header.gfx.scale.y * hitbox.hurtboxHeight
	obj.hitboxDownOffset = obj.header.gfx.scale.y * hitbox.downOffset
end

-------------------
-----Functions-----
-------------------

local sCheckpointPole = {
interactType      = INTERACT_POLE,
downOffset        = 0,
damageOrCoinValue = 0,
health            = 0,
numLootCoins      = 0,
radius            = 80,
height            = 600,
hurtboxRadius     = 0,
hurtboxHeight     = 0,
}

function bhv_checkpoint_pole_init(o)
obj_set_hitbox(o, sCheckpointPole)
end

bhvCheckpointPole = hook_behavior(nil, OBJ_LIST_GENACTOR, false, bhv_checkpoint_pole_init, nil)

------ Checkpoint flag ------

---@type Object
local last_touched_checkpoint = nil
local checkpoint_data = {}
local checkpoint_states = {}
local players_on_checkpoint = {}

local sCheckpoint = {
    interactType      = INTERACT_COIN,
    downOffset        = 0,
    damageOrCoinValue = 0,
    health            = 0,
    numLootCoins      = 0,
    radius            = 80,
    height            = 600,
    hurtboxRadius     = 0,
    hurtboxHeight     = 0,
}

function bhv_checkpoint_flag_init(o)
    obj_set_model_extended(o, E_MODEL_CHECKPOINT)
    obj_set_hitbox(o, sCheckpoint)
    checkpoint_states[o] = true
    spawn_non_sync_object(bhvCheckpointPole, E_MODEL_NONE, o.oPosX, o.oPosY, o.oPosZ, nil)

    local np = gNetworkPlayers[0]
    local data = checkpoint_data[np.currLevelNum]
    if data then
        local dx = o.oPosX - data.x
        local dy = o.oPosY - data.y
        local dz = o.oPosZ - data.z
        if dx * dx + dy * dy + dz * dz < 2500 then
            obj_set_model_extended(o, E_MODEL_CHECKPOINT2)
            checkpoint_states[o] = false
            last_touched_checkpoint = o
        end
    end
end

---@param o Object
function bhv_checkpoint_flag_loop(o)
    for i = 0, MAX_PLAYERS - 1 do
        local m = gMarioStates[i]
        if m and m.marioObj then
            local overlap = obj_check_hitbox_overlap(o, m.marioObj)

            if overlap and players_on_checkpoint[i] ~= o then
                players_on_checkpoint[i] = o

                if not checkpoint_states[o] then return end

                if last_touched_checkpoint and last_touched_checkpoint ~= o then
                    checkpoint_states[last_touched_checkpoint] = true
                    obj_set_model_extended(last_touched_checkpoint, E_MODEL_CHECKPOINT)
                end

                last_touched_checkpoint = o
                checkpoint_states[o] = false

                local np = gNetworkPlayers[0]
                checkpoint_data[np.currLevelNum] = {
                    x = o.oPosX,
                    y = o.oPosY,
                    z = o.oPosZ,
                }

                obj_set_model_extended(o, E_MODEL_CHECKPOINT2)
                play_sound(SOUND_MENU_CHANGE_SELECT + (1 << 16), { x = o.oPosX, y = o.oPosY, z = o.oPosZ })
                spawn_non_sync_object(id_bhvSparkle, E_MODEL_SPARKLES, o.oPosX, o.oPosY, o.oPosZ, function(s)
                    obj_scale(s, 5)
                end)

            elseif not overlap and players_on_checkpoint[i] == o then
                players_on_checkpoint[i] = nil
            end
        end
    end
end

hook_event(HOOK_ON_WARP, function()
    local m = gMarioStates[0]
    if obj_get_first_with_behavior_id(bhvCheckpoint_Flag_MOP) == nil then return end
    if m.action == ACT_TELEPORT_FADE_OUT or m.action == ACT_TELEPORT_FADE_IN then return end
    local np = gNetworkPlayers[0]
    local data = checkpoint_data[np.currLevelNum]
    if not data then return end

    vec3f_set(m.pos, data.x, data.y, data.z)
    m.action = ACT_IDLE
    m.area.camera.cutscene = 0
    m.hurtCounter = 0
    m.health = 0x880
end)